@@ -30,11 +30,8 @@
 
 #include <stdlib.h>
 #include <sys/types.h>
-#include <dirent.h>
 #include <string.h>
-#include <libxml/parser.h>
-#include <libxml/parserInternals.h>
-#include <libxml/xmlmemory.h>
+#include <glib/gmarkup.h>
 
 #include "bonobo-activation/bonobo-activation-i18n.h"
 #include "bonobo-activation/bonobo-activation-private.h"
@@ -52,6 +49,9 @@ typedef enum {
 } ParseState;
 
 typedef struct {
+        char *filename;         /* Filename of .server file, or NULL
+                                 * if parsing from memory.
+                                 */
         ParseState state;
         ParseState prev_state;
         int unknown_depth;
@@ -69,12 +69,18 @@ typedef struct {
 #define IS_ELEMENT(x) (!strcmp (name, x))
 
 static ParseInfo *
-parse_info_new (void)
+parse_info_new (const char *filename,
+                const char *host,
+                GSList    **entries)
 {
         ParseInfo *info = g_new0 (ParseInfo, 1);
 
         info->prev_state = STATE_UNKNOWN;
         info->state = STATE_START;
+
+        info->host = host;
+        info->entries = entries;
+        info->filename = g_strdup (filename);
         
         return info;
 }
@@ -82,6 +88,7 @@ parse_info_new (void)
 static void
 parse_info_free (ParseInfo *info)
 {
+        g_free (info->filename);
         g_free (info);
 }
 
@@ -118,7 +125,8 @@ od_validate (const char *iid, const char *type, const char *location)
 
 static void
 parse_oaf_server_attrs (ParseInfo      *info,
-                        const xmlChar **attrs)
+                        const gchar        **attr_names,
+                        const gchar        **attr_values)
 {
         const char *iid = NULL;
         const char *type = NULL;
@@ -129,12 +137,12 @@ parse_oaf_server_attrs (ParseInfo      *info,
 
         info->state = STATE_OAF_SERVER;
         
-        if (!attrs)
+        if (!attr_names || !attr_values)
                 return;
 
         do {
-                att = attrs[i++];
-                val = attrs[i++];
+                att = attr_names[i];
+                val = attr_values[i++];
 
                 if (att && val) {
                         if (!iid && !strcmp (att, "iid"))
@@ -157,6 +165,34 @@ parse_oaf_server_attrs (ParseInfo      *info,
                 return;
         }
 
+#ifdef G_OS_WIN32
+        /* If this data has been read from a .server file, and the
+         * path to the exe or dll starts with ../, make it relative to
+         * the location of the .server file. Very convenient, means
+         * yuo can install some software package that includes a
+         * Bonobo component in a freeestanding location, and just need
+         * to update your BONOBO_ACTIVATION_PATH so the .server file
+         * is found.
+         *
+         * In other cases, possibly replace configure-time shlib or
+         * exe location with the actual installed one. This is for
+         * components that have been built with the same
+         * configure-time prefix as libbonobo.
+         */
+        if (!strcmp (type, "exe") || !strcmp (type, "shlib")) {
+                if (info->filename != NULL &&
+                    !strncmp (location, "..", 2) &&
+                    G_IS_DIR_SEPARATOR (location[2])) {
+                            gchar *dirname = g_path_get_dirname (info->filename);
+
+                            location = g_build_filename (dirname, location, NULL);
+                            g_free (dirname);
+                    } else {
+                              location = _bonobo_activation_win32_replace_prefix (_bonobo_activation_win32_get_prefix (), location);
+                    }
+        }
+#endif
+
         /* Now create the ServerInfo object */
         info->cur_server = g_new0 (Bonobo_ServerInfo, 1);
 
@@ -166,6 +202,11 @@ parse_oaf_server_attrs (ParseInfo      *info,
         info->cur_server->hostname = CORBA_string_dup (info->host);
         info->cur_server->username = CORBA_string_dup (g_get_user_name ());
         info->cur_server->domain = CORBA_string_dup ("unused");
+
+#ifdef G_OS_WIN32
+        if (!strcmp (type, "exe") || !strcmp (type, "shlib"))
+                g_free ((char *) location);
+#endif
 }
 
 static GHashTable *interesting_locales = NULL;
@@ -173,16 +214,17 @@ static GHashTable *interesting_locales = NULL;
 void
 add_initial_locales (void)
 {
-        const GList *l;
+        const char * const * langs;
+        int i;
 
         if (!interesting_locales)
                 interesting_locales = g_hash_table_new (
                         g_str_hash, g_str_equal);
 
-	for (l = bonobo_activation_i18n_get_language_list (NULL);
-             l; l = l->next)
+        langs = g_get_language_names ();
+	for (i = 0; langs[i] != NULL; i++)
 		g_hash_table_insert (interesting_locales,
-                                     g_strdup ((char *) l->data),
+                                     g_strdup (langs[i]),
                                      GUINT_TO_POINTER (1));
 }
 
@@ -240,7 +282,8 @@ od_string_to_boolean (const char *str)
 
 static void
 parse_oaf_attribute (ParseInfo     *info,
-                     const xmlChar **attrs)
+                     const gchar        **attr_names,
+                     const gchar        **attr_values)
 {
         int i = 0;
         const char *type = NULL;
@@ -252,12 +295,12 @@ parse_oaf_attribute (ParseInfo     *info,
 
         info->state = STATE_OAF_ATTRIBUTE;
         
-        if (!attrs)
+        if (!attr_names || !attr_values)
                 return;
 
         do {
-                att = attrs[i++];
-                val = attrs[i++];
+                att = attr_names[i];
+                val = attr_values[i++];
                 
                 if (att && val) {
                         if (!strcmp (att, "type"))
@@ -312,18 +355,19 @@ parse_oaf_attribute (ParseInfo     *info,
 
 static void
 parse_stringv_item (ParseInfo     *info,
-                    const xmlChar **attrs)
+                    const gchar        **attr_names,
+                    const gchar        **attr_values)
 {
         const char *value = NULL;
         const char *att, *val;
         int i = 0;
 
-        if (!attrs)
+        if (!attr_names || !attr_values)
                 return;
 
         do {
-                att = attrs[i++];
-                val = attrs[i++];
+                att = attr_names[i];
+                val = attr_values[i++];
                 
                 if (att && val) {
                         if (!value && !strcmp (att, "value")) {
@@ -343,10 +387,15 @@ parse_stringv_item (ParseInfo     *info,
 }
 
 static void
-od_StartElement (ParseInfo     *info,
-                 const xmlChar *name,
-                 const xmlChar **attrs)
+od_start_element (GMarkupParseContext *context,
+                  const gchar         *name,
+                  const gchar        **attribute_names,
+                  const gchar        **attribute_values,
+                  gpointer             user_data,
+                  GError             **error)
 {
+        ParseInfo *info = user_data;
+
         switch (info->state) {
         case STATE_START:
                 if (IS_ELEMENT ("oaf_info")) 
@@ -359,7 +408,7 @@ od_StartElement (ParseInfo     *info,
                 break;
         case STATE_OAF_INFO:
                 if (IS_ELEMENT ("oaf_server")) 
-                        parse_oaf_server_attrs (info, attrs);
+                        parse_oaf_server_attrs (info, attribute_names, attribute_values);
                 else {
                         info->prev_state = info->state;
                         info->state = STATE_UNKNOWN;
@@ -368,7 +417,7 @@ od_StartElement (ParseInfo     *info,
                 break;
         case STATE_OAF_SERVER:
                 if (IS_ELEMENT ("oaf_attribute")) 
-                        parse_oaf_attribute (info, attrs);
+                        parse_oaf_attribute (info, attribute_names, attribute_values);
                 else {
                         info->prev_state = info->state;
                         info->state = STATE_UNKNOWN;
@@ -377,7 +426,7 @@ od_StartElement (ParseInfo     *info,
                 break;
         case STATE_OAF_ATTRIBUTE:
                 if (IS_ELEMENT ("item"))
-                        parse_stringv_item (info, attrs);
+                        parse_stringv_item (info, attribute_names, attribute_values);
                 else {
                         info->prev_state = info->state;
                         info->state = STATE_UNKNOWN;
@@ -411,9 +460,12 @@ add_entry (ParseInfo *info)
 }
 
 static void
-od_EndElement (ParseInfo     *info,
-               const xmlChar *name)
+od_end_element (GMarkupParseContext *context,
+                const gchar         *name,
+                gpointer             user_data,
+                GError             **error)
 {
+        ParseInfo *info = user_data;
 
         switch (info->state) {
         case STATE_ITEM:
@@ -486,122 +538,52 @@ od_EndElement (ParseInfo     *info,
         }
 }
 
-static xmlEntityPtr
-od_GetEntity (ParseState *ps, const xmlChar *name)
-{
-	return xmlGetPredefinedEntity (name);
-}
-
 static void
-od_Warning (ParseInfo *ps,
-            const char *msg,
-            ...)
+od_error (GMarkupParseContext *context,
+          GError              *error,
+          gpointer             user_data)
 {
-	va_list args;
+        ParseInfo *info = user_data;
 
-	va_start (args, msg);
-	g_logv   ("XML", G_LOG_LEVEL_WARNING, msg, args);
-	va_end   (args);
+        g_error ("Failed to parse: '%s' in file '%s'",
+                 error ? error->message : "<nomsg>",
+                 info->filename ? info->filename : "<memory>");
 }
 
-static void
-od_Error (ParseInfo *ps, const char *msg, ...)
-{
-	va_list args;
-
-	va_start (args, msg);
-	g_logv   ("XML", G_LOG_LEVEL_CRITICAL, msg, args);
-	va_end   (args);
-}
-
-static void
-od_FatalError (ParseInfo *ps, const char *msg, ...)
-{
-	va_list args;
-
-	va_start (args, msg);
-	g_logv   ("XML", G_LOG_LEVEL_ERROR, msg, args);
-	va_end   (args);
-}
-
-static xmlSAXHandler od_SAXParser = {
-	NULL, /* internalSubset */
-	NULL, /* isStandalone */
-	NULL, /* hasInternalSubset */
-	NULL, /* hasExternalSubset */
-	NULL, /* resolveEntity */
-        (getEntitySAXFunc) od_GetEntity, /* getEntity */
-	NULL, /* entityDecl */
-	NULL, /* notationDecl */
-	NULL, /* attributeDecl */
-	NULL, /* elementDecl */
-	NULL, /* unparsedEntityDecl */
-	NULL, /* setDocumentLocator */
-	NULL, /* startDocument */
-	(endDocumentSAXFunc) NULL, /* endDocument */
-	(startElementSAXFunc) od_StartElement, /* startElement */
-	(endElementSAXFunc) od_EndElement, /* endElement */
-	NULL, /* reference */
-	NULL, /* characters */
-	NULL, /* ignorableWhitespace */
-	NULL, /* processingInstruction */
-	NULL, /* comment */
-	(warningSAXFunc) od_Warning, /* warning */
-	(errorSAXFunc) od_Error, /* error */
-	(fatalErrorSAXFunc) od_FatalError, /* fatalError */
+static GMarkupParser od_gmarkup_parser = {
+        od_start_element,
+        od_end_element,
+        NULL,
+        NULL,
+        od_error
 };
 
-static void
-od_load_context (xmlParserCtxt *ctxt,
-                 GSList    **entries,
-                 const char *host)
-{
-        ParseInfo *info;
-	xmlSAXHandlerPtr oldsax;
-        int ret = 0;
-
-        info = parse_info_new ();
-        info->host = host;
-        info->entries = entries;
-
-        oldsax = ctxt->sax;
-        ctxt->sax = &od_SAXParser;
-        ctxt->userData = info;
-        /* Magic to make entities work as expected */
-	ctxt->replaceEntities = TRUE;
-
-        xmlParseDocument (ctxt);
-
-        if (ctxt->wellFormed)
-                ret = 0;
-        else {
-                if (ctxt->errNo != 0)
-                        ret = ctxt->errNo;
-                else
-                        ret = -1;
-        }
-        ctxt->sax = oldsax;
-
-        parse_info_free (info);
-
-        if (ret < 0) {
-                g_warning (_("Could not parse badly formed XML document %s"), ctxt->input->filename);
-                return;
-        }
-}
-
 static void
 od_load_file (const char *file,
               GSList    **entries,
               const char *host)
 {
-        xmlParserCtxt *ctxt;
+        gsize length;
+        gchar *contents = NULL;
+        ParseInfo *info;
+        GMarkupParseContext *ctxt = NULL;
 
-        ctxt = xmlCreateFileParserCtxt (file);
-        if (!ctxt)
-                return;
-        od_load_context (ctxt, entries, host);
-        xmlFreeParserCtxt (ctxt);
+        if (!g_file_get_contents (file, &contents, &length, NULL))
+                goto err;
+
+        info = parse_info_new (file, host, entries);
+        ctxt = g_markup_parse_context_new (&od_gmarkup_parser, 0, info,
+                                           (GDestroyNotify) parse_info_free);
+
+        if (!g_markup_parse_context_parse (ctxt, contents, length, NULL)) {
+  err:
+                g_warning (_("Could not parse badly formed XML document %s"), file);
+        }
+        if (ctxt)
+                g_markup_parse_context_free (ctxt);
+
+	if (contents)
+		g_free (contents);
 }
 
 void
@@ -609,13 +591,17 @@ bonobo_parse_server_info_memory (const char *server_info,
                                  GSList    **entries,
                                  const char *host)
 {
-        xmlParserCtxt *ctxt;
+        ParseInfo *info;
+        GMarkupParseContext *ctxt;
 
-        ctxt = xmlCreateMemoryParserCtxt (server_info, strlen (server_info));
-        if (!ctxt)
-                return;
-        od_load_context (ctxt, entries, host);
-        xmlFreeParserCtxt (ctxt);
+        info = parse_info_new (NULL, host, entries);
+
+        ctxt = g_markup_parse_context_new (&od_gmarkup_parser, 0, info,
+                                           (GDestroyNotify) parse_info_free);
+
+        if (!g_markup_parse_context_parse (ctxt, server_info, strlen (server_info), NULL))
+                g_warning ("Failed to parse serverinfo from memory");
+        g_markup_parse_context_free (ctxt);
 }
 
 static gboolean
@@ -634,23 +620,23 @@ od_load_directory (const char *directory,
                    GSList    **entries,
                    const char *host)
 {
-	DIR *directory_handle;
-	struct dirent *directory_entry;
+	GDir *directory_handle;
+	const char *directory_entry;
         char *pathname;
 
         g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, _("Trying dir %s"), directory);
 
-        directory_handle = opendir (directory);
+        directory_handle = g_dir_open (directory, 0, NULL);
 
         if (directory_handle == NULL) {
                 /* FIXME */
                 return;
         }
         
-        for (directory_entry = readdir (directory_handle);
+        for (directory_entry = g_dir_read_name (directory_handle);
              directory_entry != NULL;
-             directory_entry = readdir (directory_handle)) {
-                pathname = g_strdup_printf ("%s/%s", directory, directory_entry->d_name);
+             directory_entry = g_dir_read_name (directory_handle)) {
+                pathname = g_build_filename (directory, directory_entry, NULL);
 
                 if (od_filename_has_extension (pathname, ".server")) {
                         od_load_file (pathname, entries, host);
@@ -659,7 +645,7 @@ od_load_directory (const char *directory,
 		g_free (pathname);
         }
 
-        closedir (directory_handle);
+        g_dir_close (directory_handle);
 }
 
 
