@@ -104,7 +104,7 @@ bonobo_marshal_BOXED__STRING_BOOLEAN_DUMMY_BOXED (GClosure     *closure,
 			     g_value_get_boxed (param_values + 3),
 			     data2);
 
-	g_value_set_boxed_take_ownership (return_value, v_return);
+	g_value_take_boxed (return_value, v_return);
 }
 
 /*
@@ -169,7 +169,7 @@ bonobo_item_handler_finalize (GObject *object)
 			g_closure_unref (handler->priv->get_object);
 
 		g_free (handler->priv);
-		handler->priv = 0;
+		handler->priv = NULL;
 	}
 
 	bonobo_item_handler_parent_class->finalize (object);
